import base64
import hashlib
import hmac
import logging
import time
from enum import Enum
from urllib import parse
import json
import traceback
from datetime import datetime
import aiohttp

logger = logging.getLogger(__name__)


class MessageMethod(Enum):
    """
    发送消息方式，目前只有钉钉，后续可能会有其他形式的消息
    """
    DINGDING = 'DINGDING'


# 钉钉
class Dingding:
    def __init__(self, robot_id, secret):
        """
        :param robot_id:  你的access_token，即webhook地址中那段access_token。
        :param secret: 你的secret，即安全设置加签当中的那个密钥
        """
        self.robot_id = robot_id
        self.secret = secret

    def cal_timestamp_sign(self):
        # 根据钉钉开发文档，修改推送消息的安全设置https://ding-doc.dingtalk.com/doc#/serverapi2/qf2nxq
        # 也就是根据这个方法，不只是要有robot_id，还要有secret
        # 当前时间戳，单位是毫秒，与请求调用时间误差不能超过1小时
        # python3用int取整
        secret = self.secret
        timestamp = int(round(time.time() * 1000))
        # 密钥，机器人安全设置页面，加签一栏下面显示的SEC开头的字符串
        secret_enc = bytes(secret.encode('utf-8'))
        string_to_sign = '{}\n{}'.format(timestamp, secret)
        string_to_sign_enc = bytes(string_to_sign.encode('utf-8'))
        hmac_code = hmac.new(secret_enc, string_to_sign_enc, digestmod=hashlib.sha256).digest()
        # 得到最终的签名值
        sign = parse.quote_plus(base64.b64encode(hmac_code))
        return str(timestamp), str(sign)

    # 发送钉钉消息
    async def send_dingding_msg(self, content):
        try:
            robot_id = self.robot_id
            msg = {
                "msgtype": "text",
                "text": {"content": content + '\n' + datetime.now().strftime("%m-%d %H:%M:%S")}}
            headers = {"Content-Type": "application/json;charset=utf-8"}
            # https://oapi.dingtalk.com/robot/send?access_token=XXXXXX&timestamp=XXX&sign=XXX
            timestamp, sign_str = self.cal_timestamp_sign()
            url = 'https://oapi.dingtalk.com/robot/send?access_token=' + robot_id + \
                  '&timestamp=' + timestamp + '&sign=' + sign_str
            body = json.dumps(msg)

            async with aiohttp.ClientSession() as session:
                async with session.post(url, data=body, headers=headers, timeout=10) as response:
                    r = await response.text()
            # requests.post(url, data=body, headers=headers, timeout=10)
            logger.debug(f'成功发送钉钉: {content}')
        except Exception as e:
            logger.warning("发送钉钉失败:", e)
            traceback.print_exc()


# 发送消息
class Message:
    def __init__(self, message_method=MessageMethod.DINGDING, init_config={}):
        self.type = message_method
        self.init_config = init_config

    # 发送消息
    async def send(self, content=''):
        if self.type == MessageMethod.DINGDING:
            ding = Dingding(robot_id=self.init_config['robot_id'], secret=self.init_config['secret'])
            return await ding.send_dingding_msg(content)


ding_ding = None
msg_head = None


def set_dingding(config, head=None):
    global ding_ding, msg_head
    ding_ding = Message(MessageMethod.DINGDING, config)
    msg_head = head


async def dinding_send(msg):
    global ding_ding, msg_head
    if ding_ding:
        await ding_ding.send(msg if msg_head is None else f"{msg_head}: {msg}")
    else:
        logger.warning("请先 set_dingding 进行初始化")


# 企业微信通知
# wechat_webhook_url = 'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key=0b0....'
async def send_msg_q_wechat(url, content, proxy=None):
    try:
        data = {
            "msgtype": "text",
            "text": {
                "content": content + '\n' + datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            }
        }
        async with aiohttp.ClientSession() as session:
            async with session.post(url, data=json.dumps(data), timeout=10,
                                    proxy=proxy) as response:
                print(f'调用企业微信接口返回： {await response.text()}')
                print('成功发送企业微信')
    except Exception as e:
        print(f"发送企业微信失败:{e}")
        print(traceback.format_exc())

# async def main():
#     message_instance = Message(MessageMethod.DINGDING, {
#         'robot_id': 'e02d79d08f29878bf1351299091143cf781488dd58301ed8dde74e4ee9d9d91a',
#         'secret': 'SEC4efd80819c5ecfefa93adcd214478292724b1a7558d2d2c1eca73afdd1eb8419'
#     })
#     await message_instance.send('哈哈哈')
#
# if __name__ == '__main__':
#     asyncio.run(main())



#!/usr/bin/env python
from setuptools import setup, find_packages

install_requires = [
    'PyYAML',
    'aiofiles',
]

setup(
    name='trade-lib',
    description="我的交易库",
    version='0.1.3',
    py_modules=['trade_lib'],
    packages=find_packages(exclude=["tests"]),
    install_requires=install_requires,
)



import asyncio
import logging
import logging.config
import os
from functools import lru_cache
from pathlib import Path

import yaml
from yaml import Loader, load

logger = logging.getLogger(__name__)


def chunks(lst, n):
    """Yield successive n-sized chunks from lst."""
    for i in range(0, len(lst), n):
        yield lst[i:i + n]


def read_config(config_file):
    logger.info(f"config file: {config_file}")
    if os.path.isfile(config_file) and os.access(config_file, os.R_OK):
        with open(config_file, 'r') as f:
            return load(f, Loader=Loader)
    else:
        logger.error(f"{config_file} 文件不存在或不可读")
        exit(1)


def logging_config(config_file='logging.yml'):
    with open(config_file, 'r') as f_conf:
        dict_conf = yaml.safe_load(f_conf)
    logging.config.dictConfig(dict_conf)


def user_home_path():
    return Path.home()


def config_dir_path(dir='.config'):
    """
    返回用户当前目录下的配置目录
    :param dir:
    :return:
    """
    return os.path.join(Path.home(), dir)


@lru_cache
def read_yaml(yaml_file):
    if os.path.isfile(yaml_file) and os.access(yaml_file, os.R_OK):
        with open(yaml_file, 'r') as f:
            return load(f, Loader=Loader)
    else:
        raise Exception(f"{yaml_file} 文件不存在或不可读")


def get_exchange_config(name, file='exchanges.yaml'):
    """
    返回交易所用于 ccxt 的 api 配置
    :param name:
    :param cfg_file:
    :return:
    """
    try:
        path = config_dir_path()
        configs = read_yaml(path + '/' + file)
        return configs[name]
    except Exception as e:
        raise Exception(f"~/.config/{file} 中找不到 {name} 的 api 配置！")


async def repeat(interval, func, *args, **kwargs):
    """Run func every interval seconds.

    If func has not finished before *interval*, will run again
    immediately when the previous iteration finished.

    *args and **kwargs are passed as the arguments to func.
    """
    while True:
        try:
            await asyncio.gather(
                func(*args, **kwargs),
                asyncio.sleep(interval),
            )
        except asyncio.CancelledError:
            # logger.debug('repeat 退出')
            return
        except Exception as e:
            logger.exception('repeat')
            return


def dict_gets(d, keys, default=None):
    for key in keys:
        d = d.get(key, default)
        if isinstance(d, dict):
            continue
        else:
            return d
